/*
 * Copyright (C) 2003, 2004 Johnathan Burchill <jkerrb@users.sourceforge.net>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "kdar.h"

#include "archiveHandle.h"
#include "createArchiveThread.h"
#include "diffArchiveThread.h"
#include "getUserResponseThread.h"
#include "isolateArchiveThread.h"
#include "kdarCheckFileSizeEvent.h"
#include "kdarConfig.h"
#include "kdardialog.h"
#include "kdardlg.h"
#include "kdarview.h"
#include "kdarListViewItem.h"
#include "kdarPostListViewItemEvent.h"
#include "kdarSetCursorEvent.h"
#include "kdarStatusBarEvent.h"
#include "kdarWarningEvent.h"
#include "kdarTarListingEvent.h"
#include "kdarUserInteractionEvent.h"
#include "listArchiveThread.h"
#include "newarchivestoragedirdialog.h"
#include "openArchiveThread.h"
#include "populateTreeArchiveThread.h"
#include "restoreArchiveThread.h"
#include "testArchiveThread.h"
#include "units.h"

#include <kaccel.h>
#include <kapp.h>
#include <kcombobox.h>
#include <kconfig.h>
#include <kcursor.h>
#include <kdebug.h>
#include <kedittoolbar.h>
#include <kfiledialog.h>
#include <kfileitem.h>
#include <kfontrequester.h>
#include <kglobal.h>
#include <kio/netaccess.h>
#include <kkeydialog.h>
#include <klineedit.h>
#include <klocale.h>
#include <kmenubar.h>
#include <kmessagebox.h>
#include <kmimetype.h>
#include <kprinter.h>
#include <kstdaccel.h>
#include <kstdaction.h>
#include <kstatusbar.h>
#include <ktextbrowser.h>
#include <kurl.h>
#include <kurlpixmapprovider.h>
#include <kurlrequester.h>
#include <kurlrequesterdlg.h>

#include <qcheckbox.h>
#include <qcombobox.h>
#include <qdragobject.h>
#include <qfile.h>
#include <qfont.h>
#include <qheader.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlistview.h>
#include <qpaintdevicemetrics.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qradiobutton.h>
#include <qregexp.h>
#include <qsemaphore.h>
#include <qspinbox.h>
#include <qstringlist.h>
#include <qtextbrowser.h>
#include <qtimer.h>
#include <qvalidator.h>
#include <qwaitcondition.h>

// dar library include
#include <dar/erreurs.hpp>
#include <dar/libdar.hpp>
#include <dar/tools.hpp>

//For reading the magic number of the archive file:
#include <netinet/in.h>

libdar::archive *theArchive = 0;
kdarListViewItem *currentDirectory = 0;

//archiveItem is the basename of the archive
kdarListViewItem * archiveItem;

bool libdarWorking(false);
bool userAnswer(false);
bool gotUserResponse(false);

QSemaphore *mySemaphore;

//For expanding the tree list view
kdarListViewItem * myExpandChild;

kdar::kdar()
    : KMainWindow( 0, "kdar"),
	  createDlg(new kdarCreateDialog( this ) ),
	  createDlgWizard(new kdarCreateArchiveWizard),
      contextPopupMenu(new KActionMenu(i18n("What to you want to do?"), this)),
      kdarPixmapProvider(new KURLPixmapProvider),
	  createThread(new createArchiveThread( this ) ),
	  diffThread( new diffArchiveThread( this ) ),
	  isolateThread(new isolateArchiveThread( this ) ),
	  listThread( new listArchiveThread( this ) ),
	  openThread( new openArchiveThread( this ) ),
	  populateTreeThread( new populateTreeArchiveThread( this ) ),
	  restoreThread( new restoreArchiveThread( this ) ),
	  testThread( new testArchiveThread( this ) ),
	  userResponseThread( new getUserResponseThread( this ) ),
	  m_view(new kdarView(this)),
      m_printer(0),
      m_kdarconfig(new KDarConfig),
      extractDlg(new kdarExtractDialog)
{
    //Get the config object
	// accept dnd
    setAcceptDrops(true);
    m_view->kdarbase->archiveFileView->setDragEnabled(true);


    // tell the KMainWindow that this is indeed the main widget
    setCentralWidget(m_view);
	
    
	// then, setup our actions
    setupActions();

    // and a status bar
    statusBar()->show();

    // apply the saved mainwindow settings, if any, and ask the mainwindow
    // to automatically save settings if changed: window size, toolbar
    // position, icon size, etc.
    setAutoSaveSettings();

    //Set the answer callback functions for libdar
        //for warning messages -- which just get sent to the archiveTextBrowser
        libdar::set_warning_callback(&getUserResponseThread::interaction_warning_display);
        //for user interaction (yes/no responses) -- calls a KMessageBox::warningYesNo
        libdar::set_answer_callback(&getUserResponseThread::interaction_pause);
        //send the tar output to the archiveListView:
        libdar::set_op_tar_listing_callback(&getUserResponseThread::tarListing);

	//Read settings from local configuration file.
	readProperties();
	
	//Now copy the m_kdarconfig object to the create and restore config objects
	m_createConfig = new KDarConfig( this, 0, m_kdarconfig, false );
	m_restoreConfig = new KDarConfig( this, 0, m_kdarconfig, false );

	//Set the createDlg's KDarConfig pointer:
	createDlg->setKDarConfig( m_createConfig );
	//Create dialog connections
	//Connect the createDlg "configure" clicked() signal to its slot
	connect( createDlg->configureCreationSettingsPushButton, SIGNAL( clicked() ), this, SLOT( slotConfigureCreationSettings() ) );
	//Connect the createDlg save profile button clicked() signal to its slot
	connect( createDlg->saveProfilePushButton, SIGNAL( clicked() ), this, SLOT( slotSaveCreationProfile() ) );
	//Connect the createDlg restore profile button clicked() signal to its slot
	connect( createDlg->restoreProfilePushButton, SIGNAL( clicked() ), this, SLOT( slotRestoreCreationProfile() ) );
	//Connect the createDlg dryRunCheckBox toggled( bool ) signal to its slot
//	connect( createDlg->dryRunCheckBox, SIGNAL( toggled( bool ) ), this, SLOT( slotDryRun( bool ) ) );	
	//Restore dialog connections
	//Connect the extractDlg "configure" clicked() signal to its slot
	connect( extractDlg->configureRestoreSettingsPushButton, SIGNAL( clicked() ), this, SLOT( slotConfigureRestoreSettings() ) );
	//Connect the extractDlg save profile button clicked() signal to its slot
	connect( extractDlg->saveProfilePushButton, SIGNAL( clicked() ), this, SLOT( slotSaveRestoreProfile() ) );
	//Connect the extractDlg restore profile button clicked() signal to its slot
	connect( extractDlg->restoreProfilePushButton, SIGNAL( clicked() ), this, SLOT( slotRestoreRestoreProfile() ) );
	//Connect the extractDlg dryRunCheckBox toggled( bool ) signal to its slot
//	connect( extractDlg->dryRunCheckBox, SIGNAL( toggled( bool ) ), this, SLOT( slotDryRun( bool ) ) );	

	m_view->kdarbase->archiveFileView->setColumnWidthMode(0,QListView::Manual);
	m_view->kdarbase->archiveFileView->setColumnWidthMode(6,QListView::Maximum);
	m_view->kdarbase->archiveFileView->setSorting(1);

	userResponseThread->start();
	
	mySemaphore = new QSemaphore(1);
	(*mySemaphore)++;

	//set up an integer validator on the slicesize lineEdits
	QRegExp rx("[1-9][0-9]*");
	QRegExpValidator * restrictToWholeNumbers = new QRegExpValidator(rx, this);
	m_kdarconfig->sliceSizeLineEdit->setValidator( restrictToWholeNumbers );
	m_kdarconfig->firstSliceSizeLineEdit->setValidator( restrictToWholeNumbers );

	//create a QTimer for checking the filesize during creation operations
	//This is meant to be a primitive form of feedback for the user
	//Hopefully libdar will evolve to allow more detailed feedback for all
	//of its operations. The timer is started and stopped in the createThread.
	fileSizeTimer = new QTimer( this );
	connect( fileSizeTimer, SIGNAL( timeout() ), this, SLOT( slotCreationCheckFileSize() ) );
	
}

kdar::~kdar()
{
	if (isolateThread->running())
	{
		isolateThread->wait();
	}
	if (openThread->running())
	{
		openThread->wait();
	}
	if (restoreThread->running())
	{
		restoreThread->wait();
	}
	if (createThread->running())
	{
		createThread->wait();
	}
	if (diffThread->running())
	{
		diffThread->wait();
	}
	if (listThread->running())
	{
		listThread->wait();
	}
	if (populateTreeThread->running())
	{
		populateTreeThread->wait();
	}
	if (testThread->running())
	{
		testThread->wait();
	}
	saveProperties();
	if (m_kdarconfig) delete m_kdarconfig;
	if (createDlg) delete createDlg;
	if (createDlgWizard) delete createDlgWizard;
}

void kdar::myquit()
{
	if (isolateThread->running())
	{
		isolateThread->wait();
	}
	if (openThread->running())
	{
		openThread->wait();
	}
	if (restoreThread->running())
	{
		restoreThread->wait();
	}
	if (createThread->running())
	{
		createThread->wait();
	}
	if (diffThread->running())
	{
		diffThread->wait();
	}
	if (listThread->running())
	{
		listThread->wait();
	}
	if (populateTreeThread->running())
	{
		populateTreeThread->wait();
	}
	if (testThread->running())
	{
		testThread->wait();
	}
	saveProperties();
	if (m_kdarconfig) delete m_kdarconfig;
	if (createDlg) delete createDlg;
	if (createDlgWizard) delete createDlgWizard;
	kapp->quit();
}

void kdar::slotTerminateOperation()
{
	bool cancelled(false);
	QString message;
	
	if (libdarWorking)
	{
		//This method of stopping threads is not recommended!
		if (isolateThread->running())
		{
			isolateThread->terminate();
			isolateThread->wait();
			cancelled = true;
		}
		//These operations are commented out because they may cause problems when terminated.
		//if (openThread->running())
		//{
		//	openThread->terminate();
		//	openThread->wait();
		//	cancelled = true;
		//}
		if (restoreThread->running())
		{
			restoreThread->terminate();
			restoreThread->wait();
			cancelled = true;
		}
		if (createThread->running())
		{
			createThread->terminate();
			createThread->wait();
			cancelled = true;
		}
		if (diffThread->running())
		{
			diffThread->terminate();
			diffThread->wait();
			cancelled = true;
		}
		//These might cause problems upon termination too.
		//if (listThread->running())
		//{
		//	listThread->terminate();
		//	listThread->wait();
		//	cancelled = true;
		//}
		//if (populateTreeThread->running())
		//{
		//	populateTreeThread->terminate();
		//	populateTreeThread->wait();
		//	cancelled = true;
		//}
		//if (testThread->running())
		//{
		//	testThread->terminate();
		//	testThread->wait();
		//	cancelled = true;
		//}
		if (cancelled)
		{		
			libdarWorking = false;
			setCursor(KCursor::arrowCursor());
			message = "Cancelled operation.";
		}
		else
		{
			message = "Could not cancel operation.";
		}
	}
	else
	{
		message = "No operation to cancel.";
	}
	statusBar()->message(message, DURATION);
}

void kdar::setupActions()
{
	KStdAction::print(this, SLOT(filePrint()), actionCollection(), "print");
	KStdAction::quit(this, SLOT(myquit()), actionCollection(), "quit");

	m_toolbarAction = KStdAction::showToolbar(this, SLOT(optionsShowToolbar()), actionCollection());
	m_statusbarAction = KStdAction::showStatusbar(this, SLOT(optionsShowStatusbar()), actionCollection());
	m_recentFilesAction = KStdAction::openRecent( this, SLOT( slotArchiveChoose( const KURL & ) ), actionCollection(), "open_recent" );

    KStdAction::keyBindings(this, SLOT(optionsConfigureKeys()), actionCollection());
    KStdAction::configureToolbars(this, SLOT(optionsConfigureToolbars()), actionCollection());
    KStdAction::preferences(this, SLOT(optionsPreferences()), actionCollection());

    (void) new KAction(i18n("&Open archive"), "kdar", 0,
    		                  this, SLOT(slotArchiveChoose()),
                                  actionCollection(), "select_archive");
    (void) new KAction(i18n("&View"), "kdar_view", 0,
                                  this, SLOT(slotArchiveList()),
                                  actionCollection(), "list_archive");
    (void) new KAction(i18n("&Create"), "kdar_create", 0,
                                  this, SLOT(slotArchiveCreate()),
                                  actionCollection(), "create_archive");
    KAction *kdarExtract(new KAction(i18n("&Restore"), "kdar_extract", 0,
                                  this, SLOT(slotArchiveExtract()),
                                  actionCollection(), "extract_archive"));
    (void) new KAction(i18n("&Isolate"), "kdar_isolate", 0,
                                  this, SLOT(slotArchiveIsolate()),
                                  actionCollection(), "isolate_archive");
    KAction *kdarDiff(new KAction(i18n("&Diff"), "kdar_diff", 0,
                                  this, SLOT(slotArchiveDiff()),
                                  actionCollection(), "diff_archive"));
    KAction *kdarTest(new KAction(i18n("&Test"), "kdar_test", 0,
                                  this, SLOT(slotArchiveTest()),
                                  actionCollection(), "test_archive"));
	(void) new KAction(i18n("Clear &Message Window"), "eraser", 0, this, SLOT(slotClearMessages()), actionCollection(), "clear_messages");
	(void) new KAction(i18n("&Stop operation"), "stop", 0, this, SLOT(slotTerminateOperation()), actionCollection(), "stop_operation");
	
	contextPopupMenu->insert(kdarExtract);
    contextPopupMenu->insert(kdarDiff);
    contextPopupMenu->insert(kdarTest);

    createGUI();
}

void kdar::slotArchiveList( const bool loadArchive )
{
	if (libdarWorking)
	{
		return;
	}
	if (theArchive == 0)
	{
		QString archiveName = m_kdarconfig->archiveName();
		if (archiveName[0] == '/' && loadArchive )
		{
			kdDebug() << "archive name to list is " << archiveName.latin1() << endl;
			openArchive( m_kdarconfig );
		}
		else
		{
			return;
		}
	}
	listThread->setKDarConfig( m_kdarconfig );
	listThread->start();

}

void kdar::slotArchiveChoose( const KURL & name )
{
	//If the file doesn't exist, then remove it from the recent list
	if ( !QFile::exists( name.path() ) )
	{
		//remove from recent
		m_recentFilesAction->removeURL( name );
	}
	else
	{
		slotArchiveChoose( name.path() );
	}
}

void kdar::slotArchiveChoose( const QString & name )
{
	QString archiveName( name );
	//If archiveName was not passed, user chooses an archive file to open
	//Check that the archiveName file exists. Otherwise, cancel.
	if ( archiveName.isEmpty() || !QFile::exists( archiveName ) )
	{
		archiveName = KFileDialog::getOpenFileName(m_kdarconfig->archiveStorageDirectory(), (QString) "*.dar", this, (QString) "Select archive to open");
	}
	//if user cancels, do nothing
	if ( archiveName.isEmpty() ) return;
   
	if (archiveName[0] == '/')
	{
		m_kdarconfig->setArchiveName( archiveName );
		
		if ( theArchive != 0 )
		{
			closeArchive();
		}
		try
		{
			openArchive( m_kdarconfig );
			m_recentFilesAction->addURL( archiveName );
		}
		catch (libdar::Egeneric &e)
		{
			libdar::user_interaction_warning(e.get_message());
		}

	}
}

void kdar::slotArchiveCreate()
{
	if (libdarWorking)
	{
		return;
	}
	bool attemptRun(false);
//  TODO Need to rethink this dialog.
	
//	if (useCreateArchiveWizard)
//	{	
//		attemptRun = createDlgWizard->exec();
//	}
//	else
//	{
		attemptRun = createDlg->exec();
//	}
	//Put the creation code in a loop, so user can fix the settings if there
	//was a problem (i.e. the storage directory is the root directory)
	//TODO: This is probably overkill. Must be a better way.
	bool goodToGo( true );
	while ( attemptRun && !libdarWorking )
	{
/*		if (useCreateArchiveWizard)
		{
			//copy all user-entered values from the wizard into the advanced dialog:
			//root directory
			createDlg->rootDirectoryKURLRequester->lineEdit()->setText(createDlgWizard->rootDirectoryKURLRequester->lineEdit()->text());
			//archive name
			createDlg->archiveNameKURLRequester->lineEdit()->setText(createDlgWizard->archiveNameKURLRequester->lineEdit()->text());
			//diff archive checkbox
			createDlg->diffBackupCheckBox->setChecked(createDlgWizard->diffBackupCheckBox->isChecked());
			//diff archive name
			createDlg->referenceArchiveKURLRequester->lineEdit()->setText(createDlgWizard->referenceArchiveKURLRequester->lineEdit()->text());
			//command between slices 
			createDlg->commandBetweenSliceCreationKLineEdit->setText(createDlgWizard->commandBetweenSliceCreationKLineEdit->text());
			//directory filters
			//first remove all directory-filters from createDlg:
			createDlg->directoryFiltersListView->clear();
			QListViewItem * dirListViewItem = createDlgWizard->directoryFiltersListView->firstChild();
			while(dirListViewItem)
			{
				//copy the wizard file filters over to the dialog
				QString type = dirListViewItem->text(0);
				QString filter = dirListViewItem->text(1);
				(void) new QListViewItem(createDlg->directoryFiltersListView, type, filter);
				dirListViewItem = dirListViewItem->nextSibling();				
			}			
			//file filters
			//first remove all file-filters from createDlg:
			createDlg->fileFiltersListView->clear();
			QListViewItem * fileListViewItem = createDlgWizard->fileFiltersListView->firstChild();
			while(fileListViewItem)
			{
				//copy the wizard file filters over to the dialog
				QString type = fileListViewItem->text(0);
				QString filter = fileListViewItem->text(1);
				(void) new QListViewItem(createDlg->fileFiltersListView, type, filter);
				fileListViewItem = fileListViewItem->nextSibling();				
			}
			//compression filters
			//first remove all compression-filters from createDlg:
			createDlg->compressionFiltersListView->clear();
			QListViewItem * compListViewItem = createDlgWizard->compressionFiltersListView->firstChild();
			while(compListViewItem)
			{
				//copy the wizard file filters over to the dialog
				QString type = compListViewItem->text(0);
				QString filter = compListViewItem->text(1);
				(void) new QListViewItem(createDlg->compressionFiltersListView, type, filter);
				compListViewItem = compListViewItem->nextSibling();				
			}			
		}
*/
		//get the root directory (the one to backup)
		QString newArchiveDirectory=createDlg->rootDirectoryKURLRequester->lineEdit()->text();
		//if user cancels, do nothing
		if ( newArchiveDirectory.isEmpty() )
		{	
			KMessageBox::sorry(this, "You did not select a directory to backup.");
			goodToGo = false;
		}
		//do a check that the directory exists, eh!
		//if !fileexists... return;
		//get the  new archive basename
		QString archiveBaseTest = createDlg->archiveNameKURLRequester->lineEdit()->text();
		if ( goodToGo )
		{
				if ( archiveBaseTest.isEmpty() || ( archiveBaseTest.findRev( '/' ) == ( (int ) archiveBaseTest.length() -1 ) ) )
			{
				KMessageBox::sorry(this, "You did not choose a name for the archive.");
				goodToGo = false;
			}
		}
		if ( goodToGo )
		{
			//Insert full path into archiveBaseTest if necessary. 
			if (archiveBaseTest[0] != '/')
			{
				//User wants to override the default storage directory.
				KURL archiveStorageDirKURL( m_createConfig->archiveStorageDirectory() );
				archiveBaseTest = archiveStorageDirKURL.path(1) + archiveBaseTest;
			}
			//Check that the storage directory and the directory to back up are not the same
			KURL storageNameKURL( archiveBaseTest );
			KURL backupNameKURL( newArchiveDirectory );
			kdDebug() << "storage directory: " << storageNameKURL.directory().latin1() << endl;
			kdDebug() << "backup directory: " << backupNameKURL.directory().latin1() << endl;
			if ( storageNameKURL.directory() == backupNameKURL.path( -1 ) )
			{
				kdDebug() << "the directories are the same!" << endl;
				if ( m_createConfig->libdarCreateFileMask().is_covered( ( storageNameKURL.fileName() + QString( ".1." ) + QString ( EXTENSION ) ).latin1() ) )

				{
					//Display a message
					KMessageBox::sorry( this, "Your archive would be created in the root directory. If you really want to do this, you must exclude the archive from inclusion using a file filter. Otherwise, create the archive in another directory." );
					goodToGo = false;
				}
			}
		}
		
		if ( goodToGo && createDlg->diffBackupCheckBox->isChecked() )
		{
			if ( createDlg->referenceArchiveKURLRequester->lineEdit()->text().isEmpty() )
			{
				KMessageBox::sorry(this, "You did not choose a reference archive.");
				goodToGo = false;
			}
		}
		//Start the creation thread:
		if ( goodToGo )
		{
			//Set attemptRun to false so we don't execute this again!
			attemptRun = false;
			
			//Set the createConfig's newArchiveDirectory:
			m_createConfig->setDirectoryToBackup( newArchiveDirectory );
			//Set the NewArchiveName
			m_createConfig->setNewArchiveName( archiveBaseTest );
			//Set the differentialBackup status
			m_createConfig->setDifferentialBackup( createDlg->diffBackupCheckBox->isChecked() ); 
			//Set the differentialBackupArchiveName
			m_createConfig->setDifferentialBackupArchiveName( createDlg->referenceArchiveKURLRequester->lineEdit()->text() );
			
			if ( m_createConfig->differentialBackup() )
			{
				//if there is an open archive, close it
				if (theArchive != 0) closeArchive();
				openArchive( m_createConfig );
			}
			kdDebug() << "starting create thread." << endl;
			createThread->setKDarConfig( m_createConfig );
			createThread->start();
			kdDebug() << "started create thread." << endl;
		}
		else
		{
			attemptRun = createDlg->exec();
			goodToGo = true;
		}
	}

}

void kdar::slotArchiveExtract()
{
	if ( libdarWorking )
	{
		return;
	}

	if ( !theArchive )
	{
		slotArchiveChoose();
		if ( openThread->running() ) openThread->wait();
	}
	
	if ( !theArchive )
	{
		KMessageBox::sorry( this, "Could not open the archive for restoring." );
		return;
	}
	
	if ( extractDlg->exec() )
	{
		QString extractArchiveDirectory = extractDlg->extractKURLRequester->lineEdit()->text();
		//if user cancels, do nothing
		kdDebug() << "slotArchiveRestore(): extract directory is " << extractArchiveDirectory.latin1() << endl;
		if ( extractArchiveDirectory.isEmpty() || extractArchiveDirectory[0] != '/' )
		{
			//Inform user that they need to specify a directory
			KMessageBox::sorry(this, "You did not select a valid restoration directory.");
			return;
		}
		m_restoreConfig->setExtractArchiveDirectory( extractArchiveDirectory );
		m_restoreConfig->setDryRun( extractDlg->dryRunCheckBox->isChecked() );
		kdDebug() << "starting create thread." << endl;
		restoreThread->setArchiveFileView( m_view->kdarbase->archiveFileView );
		restoreThread->setKDarConfig( m_restoreConfig );
		restoreThread->start();
		kdDebug() << "started create thread." << endl;
	}
}
void kdar::slotArchiveIsolate()
{
	if (libdarWorking)
	{
		return;
	}
	//user should already have a reference archive name
	if (openThread->running()) openThread->wait();
	if (theArchive == 0) return;

	//user picks isolated archive name
	QString isoArchiveName=KFileDialog::getSaveFileName(QString::null, QString::null, this, (QString) "Select a filename for new isolation archive");
	//if user cancels, do nothing
	if ( isoArchiveName.isEmpty() ) return;
	m_kdarconfig->setIsolationArchiveName( isoArchiveName );	
	isolateThread->setKDarConfig( m_kdarconfig );
	isolateThread->start();
}

void kdar::slotArchiveDiff()
{
	if (libdarWorking)
	{
		return;
	}
	if (openThread->running()) openThread->wait();
	if (theArchive == 0) return;
	QString diffArchiveDirectory = KFileDialog::getExistingDirectory(QString::null, m_view, (QString) "Select directory to diff");
	//if user cancels, do nothing
	if ( diffArchiveDirectory.isEmpty() ) return;
	m_kdarconfig->setDiffArchiveDirectory( diffArchiveDirectory );
	diffThread->setKDarConfig( m_kdarconfig );
	diffThread->start();
}

void kdar::slotArchiveTest()
{
	if (libdarWorking)
	{
		return;
	}
	if (openThread->running()) openThread->wait();
	if (theArchive == 0) return;
	//test the archive
	testThread->setKDarConfig( m_kdarconfig );	
	testThread->start();

}

void kdar::saveProperties()
{
    // the 'config' object points to the session managed
    // config file.  anything you write here will be available
    // later when this app is restored
	kdDebug() << "setting config file params" << endl;
	m_view->kdarbase->archiveFileView->saveLayout(m_kdarconfig->globalConfig(), "KDar Layout");
	
	kdDebug() << "saving the kdarconfig settings" << endl;
	m_kdarconfig->writeGlobalSettings();

	m_recentFilesAction->saveEntries( m_kdarconfig->globalConfig() );	
}

void kdar::readProperties()
{
    // the 'config' object points to the session managed
    // config file.  this function is automatically called whenever
    // the app is being restored.  read in here whatever you wrote
    // in 'saveProperties'

	m_view->kdarbase->archiveFileView->restoreLayout(m_kdarconfig->globalConfig(), "KDar Layout");
	m_kdarconfig->readGlobalSettings();

	//Ask user for default storage directory if it doesn't exist yet...
	QString storageDir = m_kdarconfig->archiveStorageDirectory();
	if ( storageDir[0] != '/') //TODO: should probably test for an actual directory here.
	{
		newArchiveStorageDirDialog storageDirDlg;
		bool finished( false );
		while ( !finished )
		{
			if ( storageDirDlg.exec() )
			{
				storageDir = storageDirDlg.newArchiveStorageDir->lineEdit()->text();
				if ( storageDir[0] == '/' )
				{
					finished = true;
				}
			}
		}
		m_kdarconfig->setArchiveStorageDirectory( storageDir );
	}	
	m_recentFilesAction->loadEntries( m_kdarconfig->globalConfig() );
	m_kdarconfig->updateGUI();
	updateGUI();

}

void kdar::dragEnterEvent(QDragEnterEvent *event)
{
    // accept uri drops only
    kdDebug() << "kdar::dragEnterEvent reached." << endl;
    event->accept(QUriDrag::canDecode(event));
}

void kdar::dropEvent(QDropEvent *event)
{
	kdDebug() << "kdar::dropEvent reached." << endl;

    // this is a very simplistic implementation of a drop event.  we
    // will only accept a dropped URL.  the Qt dnd code can do *much*
    // much more, so please read the docs there
    QStrList uri;

    // see if we can decode a URI.. if not, just ignore it
    if ( QUriDrag::decode( event, uri ) )
    {
        // okay, we have a URI.. process it
        QString url, target;
        url = uri.first();
	KURL theURL = KURL::fromPathOrURL( url );
	// okay, somehow list this archive? or just select it?
	if ( theURL.isLocalFile() )
	{
		kdDebug() << "Local file!" << endl;
		if ( url.find(QRegExp(".[1-9][0-9]*.dar") ) >= 1)
		{
			kdDebug() << "I think it's a .dar file!" << endl;
			m_view->kdarbase->slotArchiveChoose( theURL.path() );
    		slotArchiveList();
		}
		else
		{
			kdDebug() << "Not a .dar file!" << endl;
		}
	}
	kdDebug() << "You dropped..." << url.latin1() << endl;
	kdDebug() << "The path is... " << theURL.path().latin1() << endl;
    }
}

bool kdar::event( QEvent * e )
{
	//Need a waitcondition for user interaction
	QWaitCondition userResponse;
//	kdDebug() << "Received event." << endl;
 	if ( e->type() >= QEvent::User )
	{
	    if ( e->type() == 1001 )  // It must be a kdarWarningEvent
	    {
			//kdDebug() << "kdar::event: It is a kdarWarningEvent...updating textbrowser." << endl;
			kdarWarningEvent* we = (kdarWarningEvent*)e;
			m_view->kdarbase->archiveTextBrowser->append(we->message());
			return true;
	    }
	
		if ( e->type() == 1002 )  // It must be a kdarUserInteractionEvent
	    {
			bool fileSizeTimerRunning( fileSizeTimer->isActive() );
			if ( fileSizeTimerRunning ) fileSizeTimer->stop();
			//kdDebug() << "kdar::event: It is a kdarUserInteractionEvent. Asking the question..." << endl;
			kdarUserInteractionEvent* uie = (kdarUserInteractionEvent*)e;
			int ret = KMessageBox::warningYesNo(0, uie->question());
			gotUserResponse = (ret == KMessageBox::Yes);
			//kdDebug() << "kdar::event: User answered " << gotUserResponse<< ". Wake up the interaction_pause thread." << endl;
			(*mySemaphore)--;
			//kdDebug() << "kdar::event: sent wake-up signal." << endl;
			if ( fileSizeTimerRunning ) fileSizeTimer->start( CHECKINTERVAL );
			return true;
		}
		
		if ( e->type() == 1003 )  // It must be a kdarTarListingEvent
		{
			//kdDebug() << "kdar::event: It is a kdarTarListingEvent...updating listview." << endl;
			kdarTarListingEvent* te = (kdarTarListingEvent*)e;
			kdarListViewItem *directory = te->directory();
			QString filename = te->filename();
			QString perm = te->perm();
			QString uid = te->uid();
			QString gid = te->gid();
			QString size = te->size();
			QString date = te->date();
			QString flag = te->flag();
			QPixmap icon = te->icon();
			
			if (directory == 0)
			{
				//kdDebug() << "currentDirectory == 0" << endl;
				return true;
			}
			//kdDebug() << "listing..." << filename.latin1() << endl;
			kdarListViewItem *newItem =  new kdarListViewItem(directory, filename, perm, uid, gid, size, date, flag);
			newItem->setPixmap(0, icon);
			return true;
		}
	    if ( e->type() == 1004 )  // It must be a kdarStatusBarEvent
	    {
			//kdDebug() << "kdar::event: It is a kdarStatusBarEvent...posting message." << endl;
			kdarStatusBarEvent* se = (kdarStatusBarEvent*)e;
			changeStatusbar(se->message());
			return true;
	    }
	    if ( e->type() == 1005 )  // It must be a kdarSetCursorEvent
	    {
			//kdDebug() << "kdar::event: It is a kdarSetCursorEvent...changing cursor." << endl;
			kdarSetCursorEvent* sce = (kdarSetCursorEvent*)e;
			setCursor(sce->cursor());
			return true;
	    }
	    if ( e->type() == 1006 )  // A kdarPostListViewItemEvent
	    {
			kdarPostListViewItemEvent* pe = (kdarPostListViewItemEvent*) e;
			if ( pe->clearView() )
			{
				m_view->kdarbase->archiveFileView->clear();
			}
			archiveItem = new kdarListViewItem(m_view->kdarbase->archiveFileView, pe->text());
			return true;
	    }
	    if ( e->type() == 1007 )  // A kdarCheckFileSizeEvent
	    {
			kdarCheckFileSizeEvent* ce = (kdarCheckFileSizeEvent*) e;
			if ( ce->checkFileSizeAction() == kdarCheckFileSizeEvent::STARTTIMER )
			{
				statusBar()->insertItem( "Writing slice 1... ", 0, 0, true );
				fileSizeTimer->start( CHECKINTERVAL );
			}
			else
			{
				fileSizeTimer->stop();
				statusBar()->removeItem( 0 );
			}
			return true;
	    }
	}
	return QWidget::event( e );
}

void kdar::slotClearMessages()
{
	//Clear the message window
	m_view->kdarbase->archiveTextBrowser->clear();
}

void kdar::slotConfigureCreationSettings()
{
	//Copy the createConfig instance so that we can cancel the changes if necessary
	KDarConfig * tmpConfig = new KDarConfig(this, 0, m_createConfig, false);
	if ( tmpConfig->exec() )
	{
		*m_createConfig = *tmpConfig;
	}
	delete tmpConfig;
}

void kdar::slotConfigureRestoreSettings()
{
	//Copy the restoreConfig instance so that we can cancel the changes if necessary
	KDarConfig * tmpConfig = new KDarConfig(this, 0, m_restoreConfig, false);
	if ( tmpConfig->exec() )
	{
		*m_restoreConfig = *tmpConfig;
	}
	delete tmpConfig;
}

void kdar::slotSaveCreationProfile()
{
	//Pop up a save file dialog
	QString saveFilename = KFileDialog::getSaveFileName( m_kdarconfig->archiveStorageDirectory(), QString::null, 0, "Choose a name for the profile" );
	//If successful, set the m_createConfig's KConfig to store to the new file.
	if ( !saveFilename.isEmpty() )
	{
		KConfig * newConfig = new KConfig( saveFilename, false, false ); 
		m_createConfig->setGlobalConfig( newConfig );
		m_createConfig->setDirectoryToBackup( createDlg->rootDirectoryKURLRequester->lineEdit()->text() );
		m_createConfig->setNewArchiveName( createDlg->archiveNameKURLRequester->lineEdit()->text() );
		m_createConfig->setDifferentialBackup( createDlg->diffBackupCheckBox->isChecked() );
		m_createConfig->setDifferentialBackupArchiveName( createDlg->referenceArchiveKURLRequester->lineEdit()->text() );
		m_createConfig->setDryRun( createDlg->dryRunCheckBox->isChecked() );
		m_createConfig->writeGlobalSettings();
		kdDebug() << "slotSaveCreationProfile(): saved settings." << endl;
		//Display a message that the settings were saved.
		statusBar()->message( "Settings saved.", DURATION );
	}
	else
	{
		//Display a message that the settings were not saved.
		statusBar()->message( "Settings not saved.", DURATION );
	}
}

void kdar::slotRestoreCreationProfile()
{
	//Pop up an open file dialog --- the file must exist on disk!
	QString restoreFilename = KFileDialog::getOpenFileName( m_kdarconfig->archiveStorageDirectory(), QString::null, 0, "Choose the profile to restore" );
	//If successful, set the m_createConfig's KConfig to restore from the new file.
	if ( !restoreFilename.isEmpty() )
	{
		KConfig * newConfig = new KConfig( restoreFilename, false, false ); 
		m_createConfig->setGlobalConfig( newConfig );
		m_createConfig->readGlobalSettings();
		m_createConfig->updateGUI();
		//update the createDlg's settings.
		createDlg->rootDirectoryKURLRequester->lineEdit()->setText( m_createConfig->directoryToBackup() );
		createDlg->archiveNameKURLRequester->lineEdit()->setText( m_createConfig->newArchiveName() );
		createDlg->diffBackupCheckBox->setChecked( m_createConfig->differentialBackup() );
		createDlg->referenceArchiveKURLRequester->lineEdit()->setText( m_createConfig->differentialBackupArchiveName() );
		createDlg->dryRunCheckBox->setChecked( m_createConfig->dryRun() );
		kdDebug() << "slotRestoreCreationProfile(): saved restored." << endl;
		//Display a message that the settings were restored.
		statusBar()->message( "Settings restored.", DURATION );
	}
	else
	{
		//Display a message that the settings were not restored.
		statusBar()->message( "Settings not restored.", DURATION );
	}
}

void kdar::slotSaveRestoreProfile()
{
	//Pop up a save file dialog
	QString saveFilename = KFileDialog::getSaveFileName( m_kdarconfig->archiveStorageDirectory(), QString::null, 0, "Choose a name for the profile" );
	//If successful, set the m_restoreConfig's KConfig to store to the new file.
	if ( !saveFilename.isEmpty() )
	{
		KConfig * newConfig = new KConfig( saveFilename, false, false ); 
		m_restoreConfig->setGlobalConfig( newConfig );
		m_restoreConfig->setExtractArchiveDirectory( extractDlg->extractKURLRequester->lineEdit()->text() );
		m_restoreConfig->setDryRun( extractDlg->dryRunCheckBox->isChecked() );
		m_restoreConfig->writeGlobalSettings();
		kdDebug() << "slotSaveRestoreProfile(): saved settings." << endl;
		//Display a message that the settings were saved.
		statusBar()->message( "Settings saved.", DURATION );
	}
	else
	{
		//Display a message that the settings were not saved.
		statusBar()->message( "Settings not saved.", DURATION );
	}
}

void kdar::slotRestoreRestoreProfile()
{
	//Pop up an open file dialog --- the file must exist on disk!
	QString restoreFilename = KFileDialog::getOpenFileName( m_kdarconfig->archiveStorageDirectory(), QString::null, 0, "Choose the profile to restore" );
	//If successful, set the m_restoreConfig's KConfig to restore from the new file.
	if ( !restoreFilename.isEmpty() )
	{
		KConfig * newConfig = new KConfig( restoreFilename, false, false ); 
		m_restoreConfig->setGlobalConfig( newConfig );
		m_restoreConfig->readGlobalSettings();
		m_restoreConfig->updateGUI();
		//update the extractDlg's settings.
		extractDlg->extractKURLRequester->lineEdit()->setText( m_restoreConfig->extractArchiveDirectory() );
		extractDlg->dryRunCheckBox->setChecked( m_restoreConfig->dryRun() );
		kdDebug() << "slotRestoreRestoreProfile(): restored settings." << endl;
		//Display a message that the settings were restored.
		statusBar()->message( "Settings restored.", DURATION );
	}
	else
	{
		//Display a message that the settings were not restored.
		statusBar()->message( "Settings not restored.", DURATION );
	}
}

void kdar::filePrint()
{
    // this slot is called whenever the File->Print menu is selected,
    // the Print shortcut is pressed (usually CTRL+P) or the Print toolbar
    // button is clicked
    if (!m_printer) m_printer = new KPrinter;
    if (m_printer->setup(this))
    {
        // setup the printer.  with Qt, you always "print" to a
        // QPainter.. whether the output medium is a pixmap, a screen,
        // or paper
        QPainter p;
        p.begin(m_printer);

        // we let our view do the actual printing
        QPaintDeviceMetrics metrics(m_printer);
        m_view->print(&p, m_printer, &metrics);

        // and send the result to the printer
        p.end();
    }
}

void kdar::optionsShowToolbar()
{
    // this is all very cut and paste code for showing/hiding the
    // toolbar
    if (m_toolbarAction->isChecked())
        toolBar()->show();
    else
        toolBar()->hide();
}

void kdar::optionsShowStatusbar()
{
    // this is all very cut and paste code for showing/hiding the
    // statusbar
    if (m_statusbarAction->isChecked())
        statusBar()->show();
    else
        statusBar()->hide();
}

void kdar::optionsConfigureKeys()
{
    KKeyDialog::configure(actionCollection());
}

void kdar::optionsConfigureToolbars()
{
    // use the standard toolbar editor
    saveMainWindowSettings( m_kdarconfig->globalConfig(), autoSaveGroup() );
    KEditToolbar toolDlg(actionCollection());
    connect(&toolDlg, SIGNAL(newToolbarConfig()), this, SLOT(newToolbarConfig()));
    toolDlg.exec();
}

void kdar::newToolbarConfig()
{
    // this slot is called when user clicks "Ok" or "Apply" in the toolbar editor.
    // recreate our GUI, and re-apply the settings (e.g. "text under icons", etc.)
    createGUI();
    applyMainWindowSettings(m_kdarconfig->globalConfig(), autoSaveGroup() );
}

void kdar::optionsPreferences()
{
    // popup some sort of preference dialog, here
    QString oldArchiveStorageDirectory = m_kdarconfig->archiveStorageDirectory();
	QString statusMessage( "Settings not changed." );
	if (m_kdarconfig->exec())
    {
		statusMessage = "Settings changed.";
		QString nullDir = m_kdarconfig->archiveStorageDirectory();
		if ( nullDir[0] != '/' )
		{
			//Send message that the archiveStorage Directory is unchanged.
			
			statusMessage = "Settings changed. Invalid archive storage directory: left unchanged.";
		//Put the old archiveStorageDir back into the URLRequester.
			m_kdarconfig->setArchiveStorageDirectory( oldArchiveStorageDirectory );
			m_kdarconfig->archiveStorageDirKURLRequester->lineEdit()->setText( oldArchiveStorageDirectory);	
		}
		updateGUI();
		m_kdarconfig->writeGlobalSettings();
	}
	//Send status message.
	statusBar()->message(statusMessage, DURATION);
}

void kdar::updateGUI()
{
	//Update KDar's fonts
	kapp->setFont( m_kdarconfig->generalFont() );
	m_view->kdarbase->archiveFileView->setFont( m_kdarconfig->fileBrowserFont() );
	m_view->kdarbase->archiveTextBrowser->setFont( m_kdarconfig->messageWindowFont() );
	statusBar()->setFont( m_kdarconfig->statusBarFont() );
}

void kdar::changeStatusbar(const QString& text)
{
    // display the text on the statusbar
    statusBar()->message(text);
}

void kdar::changeCaption(const QString& text)
{
    // display the text on the caption
    setCaption(text);
}

void kdar::openArchive( KDarConfig * kdconfig )
{
	if ( kdconfig == 0 )
	{
		kdconfig = m_kdarconfig;
	}
	
	kdDebug() << "libdarWorking is " << libdarWorking << endl;
	if (libdarWorking)
	{
		return;
	}
	QString archiveName = kdconfig->archiveName();
	if (archiveName[0] != '/')
	{
		if (theArchive !=0) delete theArchive;
		theArchive=0;
		return;
	}
	openThread->setKDarConfig( kdconfig );
	openThread->start();
}

void kdar::closeArchive()
{
	if (theArchive != 0)
	{
		delete theArchive;
		theArchive = 0;
		kdDebug() << "Archive closed." << endl;
	}
}

void kdar::slotCreationCheckFileSize()
{
	//Stop the timer
	fileSizeTimer->stop();
	//Check the filesize
	long i ( lastArchiveSliceNumber( m_createConfig->newArchiveName() ) );
	QFile newArchiveFile( m_createConfig->newArchiveName() + QString( "." ) + QString::number( i ) + QString( "." ) + EXTENSION );
	if ( newArchiveFile.exists() )
	{
		long sizeMB( newArchiveFile.size()/1048576 );
		statusBar()->changeItem( QString( "Writing slice " ) + QString::number( i ) + QString( "... " ) + QString::number( sizeMB ) + QString (" MB" ), 0 );
	}
	else
	{
		kdDebug() << "kdar::slotCreationCheckFileSize(): " << m_createConfig->newArchiveName().latin1() << "does not exist." << endl;
	}
	//Start timer again
	fileSizeTimer->start( CHECKINTERVAL );
}

long kdar::lastArchiveSliceNumber( const QString & archiveBaseName )
{
	//Find out the number of the slice that is being written by libdar::op_create	
	//This is the last slice number.
	long i( 1 );
	QFile currentFile( archiveBaseName + QString( "." ) + QString::number( i ) + QString( "." ) + EXTENSION ) ;
	bool lastSlice( false );
	long lastSliceNumber ( 0 ); // will return 0 if no valid last slice is found
	while ( currentFile.exists() && !lastSlice )
	{
		//Check whether the slice is a terminating slice:
		//Open the file
		libdar::U_32 magic;
		libdar::U_32 tmp;
		char label[10];
		char flag;
		if ( currentFile.open( IO_ReadOnly ) )
		{
			//read the magic number
			currentFile.readBlock( ( char * )&tmp, sizeof( libdar::U_32 ) );
			magic = ntohl( tmp );
			//read the internal label (see libdar/header.hpp)
			currentFile.readBlock( label, 10 );
			//read the terminating flag
			currentFile.readBlock( &flag, 1 );
			//close the file
			currentFile.close();
		}
		if ( flag == 'T' )
		{
			//This is the last slice.
			lastSlice = true;
			lastSliceNumber = i;
		}
		currentFile.setName( archiveBaseName + QString( "." ) + QString::number( ++i ) + QString( "." ) + EXTENSION );
	}
	return lastSliceNumber;
}

#include "kdar.moc"
