/***************************************************************************
 *   Copyright (C) 2004 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "listArchiveThread.h"

#include "archiveHandle.h"
#include "kdar.h"
#include "kdarConfig.h"
#include "kdardialog.h"
#include "kdarListViewItem.h"
#include "kdarPostListViewItemEvent.h"
#include "kdarSetCursorEvent.h"
#include "kdarStatusBarEvent.h"
#include "kdarWarningEvent.h"
#include "openArchiveThread.h"

#include <kcursor.h>
#include <kiconloader.h>
#include <klistview.h>

#include <qstring.h>
#include <qpixmap.h>

#include <dar/libdar.hpp>
#include <dar/tools.hpp>
#include <dar/erreurs.hpp>

#include <string>

extern bool libdarWorking;
extern kdarListViewItem *archiveItem;
extern kdarListViewItem *currentDirectory;

extern libdar::archive *theArchive;

listArchiveThread::listArchiveThread( kdar * kdarWidget )
{
	widget = kdarWidget;
}
 
listArchiveThread::~listArchiveThread()
{
}

void listArchiveThread::run()
{
	libdarWorking = true;
	kdarSetCursorEvent *setWaitCursor = new kdarSetCursorEvent( KCursor::waitCursor());
    QApplication::postEvent( widget, setWaitCursor);
	//this is needed so that the archive gets opened
	//before we use it. Bad hack, as it makes UI thread
	//unresponsive
	if (widget->openThread->running()) widget->openThread->wait();

	//call the libdar listing function
	libdar::et_mask selection;// = new et_mask();
	selection.add_mask(libdar::bool_mask(true));
	
	archiveHandle archiveHndl( m_kdc->archiveName() );
	
	kdarPostListViewItemEvent *postListViewEvent = new kdarPostListViewItemEvent( archiveHndl.getArchiveBasename(), true );
	QApplication::postEvent( widget, postListViewEvent );
	
	//set kdar pixmap
	QPixmap smallIcon = SmallIcon("kdar");
	archiveItem->setPixmap(0, smallIcon);
	//Grab the top-level directory items
	try
	{
		currentDirectory = archiveItem;
		libdar::get_children_of(theArchive, "");
		currentDirectory = 0;
	}
	catch (libdar::Egeneric & e)
	{
		libdar::user_interaction_warning(e.get_message());
		libdarWorking = false;
	}
	kdarSetCursorEvent *setArrowCursor = new kdarSetCursorEvent( KCursor::arrowCursor());
    QApplication::postEvent( widget, setArrowCursor);

	libdarWorking = false;
}

KDarConfig * listArchiveThread::kdarConfig() const
{
	return m_kdc;
}

void listArchiveThread::setKDarConfig( KDarConfig * kdarconfig )
{
	m_kdc = kdarconfig;
}
