/***************************************************************************
 *   Copyright (C) 2004 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "openArchiveThread.h"

#include "archiveHandle.h"
#include "kdar.h"
#include "kdarConfig.h"
#include "kdardialog.h"
#include "kdarListViewItem.h"
#include "kdarPostListViewItemEvent.h"
#include "kdarSetCursorEvent.h"
#include "kdarStatusBarEvent.h"

#include <kcursor.h>
#include <kdebug.h>

#include <qstring.h>

#include <dar/libdar.hpp>
#include <dar/erreurs.hpp>

#include <string>

extern bool libdarWorking;
extern libdar::archive *theArchive;
extern kdarListViewItem *archiveItem;

openArchiveThread::openArchiveThread( kdar * kdarWidget )
{
	widget = kdarWidget;
}

openArchiveThread::~openArchiveThread()
{
}

void openArchiveThread::run()
{
	libdarWorking = true;
	kdarSetCursorEvent *setWaitCursor = new kdarSetCursorEvent( KCursor::waitCursor() );
    QApplication::postEvent( widget, setWaitCursor );
	kdarStatusBarEvent *statusEventBegin = new kdarStatusBarEvent( "Opening archive..." );	
	QApplication::postEvent( widget, statusEventBegin );

	//check whether we have a valid archiveName:
	
	archiveHandle archiveHndl( m_kdc->archiveName() );
//	*ref_root = archiveHndl.getArchiveLibdarPath();
//	ref_filename = archiveHndl.getArchiveBasename().latin1();
	try
	{
		kdDebug() << "opening the archive" << endl;
		theArchive = new libdar::archive(archiveHndl.getArchiveLibdarPath(), archiveHndl.getArchiveBasename().latin1(), EXTENSION, libdar::crypto_none, m_kdc->passwordString().latin1(), m_kdc->inputPipe().latin1(), m_kdc->outputPipe().latin1(), m_kdc->executeRef().latin1(), m_kdc->verbose() );
		kdarPostListViewItemEvent *postListViewEvent = new kdarPostListViewItemEvent( archiveHndl.getArchiveBasename(), true );
    	QApplication::postEvent( widget, postListViewEvent );
		kdDebug() << "finished opening archive" << endl;
	}
	catch (libdar::Egeneric &e)
	{
		kdDebug() << "problem opening archive" << endl;
		libdar::user_interaction_warning(e.get_message());
		theArchive = 0;
		kdDebug() << "caught problem opening archive" << endl;
		libdarWorking = false;
	}
	kdDebug() << "openArchive: returning" << endl;
	kdarSetCursorEvent *setArrowCursor = new kdarSetCursorEvent( KCursor::arrowCursor());
    QApplication::postEvent( widget, setArrowCursor);
	kdarStatusBarEvent *statusEventEnd = new kdarStatusBarEvent( "Opening archive...done." );	
	QApplication::postEvent( widget, statusEventEnd );
	libdarWorking = false;
}

void openArchiveThread::setKDarConfig( KDarConfig * kdarconfig )
{
	m_kdc = kdarconfig;
}
