/***************************************************************************
 *   Copyright (C) 2004 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "populateTreeArchiveThread.h"

#include "kdar.h"
#include "kdarConfig.h"
#include "kdardialog.h"
#include "kdarListViewItem.h"
#include "kdarSetCursorEvent.h"
#include "kdarStatusBarEvent.h"
#include "kdarWarningEvent.h"

#include <kcursor.h>
#include <kdebug.h>
#include <kiconloader.h>
#include <klistview.h>

#include <qstring.h>
#include <qpixmap.h>

#include <dar/libdar.hpp>
#include <dar/tools.hpp>
#include <dar/erreurs.hpp>

#include <string>

extern bool libdarWorking;
extern kdarListViewItem *archiveItem;
extern kdarListViewItem *currentDirectory;
extern kdarListViewItem *myExpandChild;

extern libdar::archive *theArchive;

populateTreeArchiveThread::populateTreeArchiveThread( kdar * kdarWidget )
{
	widget = kdarWidget;
}

populateTreeArchiveThread::~populateTreeArchiveThread()
{
}

void populateTreeArchiveThread::run()
{
	libdarWorking = true;
	kdarSetCursorEvent *setWaitCursor = new kdarSetCursorEvent( KCursor::waitCursor());
    QApplication::postEvent( widget, setWaitCursor);
	while( myExpandChild ) {
		if (!myExpandChild->firstChild())
		{
			QString myExpandDirectory;
			try
			{
				currentDirectory = myExpandChild;
				kdDebug() << "currentDirectory is " << currentDirectory->text(0).latin1() << endl;
				//We have to climb the listview tree to get the
				//relative path of the directory:
		       	kdarListViewItem *parent = myExpandChild->parent();
		       	kdarListViewItem *tmpItem = myExpandChild;
		       	while (parent)
				{
					if (tmpItem->parent())
					{
						myExpandDirectory.prepend(QString("/")+tmpItem->text(0));
						tmpItem = tmpItem->parent();
					}
					parent = tmpItem->parent();
		       	}
				myExpandDirectory = myExpandDirectory.right(myExpandDirectory.length()-1);
				kdDebug() << "Populating tree with " << myExpandDirectory.latin1() << endl;
				//cache this for determing the icon type:
				widget->setExpandDirectory(myExpandDirectory);
				//Only populate the directory list if it has not
				//already been populated.
				//I suppose that if the directory list has a firstChild()
				//then it has been populated.
				libdar::get_children_of(theArchive, myExpandDirectory.latin1());
				currentDirectory = 0;
			}
			catch(libdar::Egeneric &e)
			{
				libdar::user_interaction_warning(e.get_message());
				libdarWorking = false;
			}
		}
		myExpandChild = myExpandChild->nextSibling();
	}
	kdarSetCursorEvent *setArrowCursor = new kdarSetCursorEvent( KCursor::arrowCursor());
    QApplication::postEvent( widget, setArrowCursor);
	libdarWorking = false;
}
